@@ -9,7 +9,16 @@
 #'
 #' @noRd
 restruct_ssn_missing <- function(ssn.object, observed_index, missing_index) {
-  if (length(missing_index) > 0) {
+  if (length(missing_index) > 0) { # used to be length when which() was used
+    # only return .missing object if necessary
+    ssn.object$preds$.missing <- ssn.object$obs[missing_index, , drop = FALSE]
+  }
+  ssn.object$obs <- ssn.object$obs[observed_index, , drop = FALSE]
+  ssn.object
+}
+
+restruct_ssn_missing_bigdata <- function(ssn.object, observed_index, missing_index) {
+  if (sum(missing_index) > 0) { # used to be length when which() was used
     # only return .missing object if necessary
     ssn.object$preds$.missing <- ssn.object$obs[missing_index, , drop = FALSE]
   }
